/**
 * 会话模块
 * @author yutent<yutent.io@gmail.com>
 * @date 2020/09/24 11:18:23
 */

import { uuid, sha1 } from 'crypto.js'

import Store from './lib/redis-store.js'

const DEFAULT_CONFIG = {
  ttl: 3600 * 24 * 7,
  domain: '', // NODESSID域
  level: 0, // 校验级别, 0: 不校验客户端, 2: ua, 4: ip, 6: ua + ip
  secret: 'it_is_secret_key', // jwt密钥, 使用时请修改
  db: {
    host: '127.0.0.1',
    port: 6379,
    db: 0
  }
}

function sessionMiddleware(req, res, next) {
  let opt = this.get('session')
  let cache = req.cookie('NODESSID')
  let deviceID = ''
  let ssid

  // options请求不处理会话
  if (req.method === 'OPTIONS') {
    return next()
  }

  // 校验UA
  if (opt.level & 2) {
    deviceID += req.header('user-agent')
  }

  // 校验IP
  if (opt.level & 4) {
    deviceID += req.ip()
  }

  if (deviceID) {
    deviceID = sha1(deviceID)

    // ssid 最后16位是指纹
    if (cache) {
      if (cache.slice(-16) === deviceID.slice(-16)) {
        ssid = cache
      } else {
        ssid = uuid('') + deviceID.slice(-16)
      }
    }
  } else {
    ssid = cache || sha1(uuid())
  }

  res.cookie('NODESSID', ssid, {
    maxAge: opt.ttl,
    httpOnly: true,
    domain: opt.domain
  })
  // 缓存ssid到req上
  req.ssid = ssid
  this.$$session.update(ssid)

  next()
}

// 会话中间件
export function createSession() {
  return {
    name: 'session',
    install(conf = {}) {
      if (!conf.secret) {
        console.warn(
          new Error(
            'You must set a `secret` key for session, or it will use the default key.'
          )
        )
      }
      let session = Object.assign({}, DEFAULT_CONFIG, conf)
      this.set({ session })

      // 注册中间件
      this.use(sessionMiddleware)

      // 这里只创建session的存储器, 而初始化操作在中间件中进行
      return new Store(session)
    }
  }
}
